% -----------------------------------------------------------------------
% Object Recoloring based on Intrinsic Image Estimation
% S. Beigpour, J.van de Weijer
% ICCV 2011
% -----------------------------------------------------------------------
% Two-illuminant MIDR model estimation Algorithm
%       mentioned LINEs are taken from the Algorithm 1 in paper
% -----------------------------------------------------------------------
function [MbD, MsD, MbS, MsS, CC, LDc, LSc]=MIDR_Estimation(f,mask,options)

    errThr = options.errThr;
    delta = options.delta;
    maxItr = options.maxItr;
        
    load Planckian;
    specolor=Plan(6:157,2:4);
    se = strel('disk',5); % is used in the end of the main loop

    % LINE 2
    CbD=(RBRE(f,mask,options))';
    
    % LINE 3,4
    % Since we don't know the Cs yet we use the given estimation of CbLd
    % for the moment (note that we're using CIE_Method1 and not CIE_Method2
    % because we don't know Cb to then divide it by Cs as in the second
    % function)
    [mcbmcs,LD]=CIE_Method1(f,mask,CbD,specolor);  %Planckian light assumption 

    % LINE 5
    diff=sqrt(sum((f-mcbmcs).^2,3)); %checking the model fit
    dMask=(diff<errThr).*mask;  %getting the area which couldn't be modeled properly

    % Initial (previous) estimation
    LSp=[0 0 0];
    LDp=LD;

    CbD=(RBRE(f,logical(mask.*dMask),options))';
    LDc=CIE_Method3(f,mask,CbD,specolor);
    CC=CbD./LD;
    CC=CC./norm(CC)*255;
    CbS=(RBRE(f,logical(mask.*(~dMask)),options))';
    LSc=CbS./CC;
    LSc=LSc./norm(LSc)*255;

    % LINE 6-13
    while (sum(abs(LDp-LDc))>delta || sum(abs(LSp-LSc))>delta ) && maxItr

        % LINE 12
        [MsD, reconD]=CIE_Method2(f,mask,CC,LDc);
        [ ~ , reconS]=CIE_Method2(f,mask,CC,LSc);
        dMask=( sum(abs(f-reconD),3) < sum(abs(f-reconS),3) ).*mask;
        
        % LINE 7
        CbD=(RBRE(f,logical(mask.*dMask),options))';
        
        % LINE 8
        LDc=CIE_Method3(f,logical(mask.*dMask),CbD,specolor);
        
        % LINE 9
        CC=CbD./LDc;
        CC=CC./norm(CC)*255;
        
        % LINE 10
        CbS=(RBRE(f,logical(mask.*(imerode(~dMask,se))),options))';
        
        % LINE 11
        LSc=CbS./CC;
        LSc=LSc./norm(LSc)*255;

        maxItr=maxItr-1;
        LDp=LDc;
        LSp=LSc;
    end
    
    MsD = MsD.*dMask;
    
    % LINE 14
    [MbD, MbS, MsS] = recalculateMbMs(f,mask,dMask,MsD,LDc,LSc,CC);
    
    % Rescaling which we use for visualization purposes
    MbD=MbD.*norm(LDc)./norm(CC.*LDc)*255; 
    
    fprintf('The estimated Material Color is:\n [ Red: %f \t Green: %f \t Blue: %f ]\n',CC(1),CC(2),CC(3));
    fprintf('The estimated Illuminant Colors by the Bi-illuminant model are:\n[ Red: %f \t Green: %f \t Blue: %f ]\n[ Red: %f \t Green: %f \t Blue: %f ]\n',LDc(1),LDc(2),LDc(3),LSc(1),LSc(2),LSc(3));
end
